#!/usr/bin/perl -w

use strict;
use Tk;
use IO::Handle;
use Socket;

my $host = "tv";
my $port = 9798;
my $debug = 1;
my $pid;
my $iaddr = inet_aton($host);
my $paddr = sockaddr_in($port, $iaddr);
my $proto = getprotobyname('tcp');
my $now_transmitted = "";
my $me = $0;
$me =~ s#.*/##;

# open(FD, "</real-home/seife/projects/dvbserver/usr/local/etc/dvb-channels") or die "can't open dvb-channels: $!";
socket(SOCK, PF_INET, SOCK_STREAM, $proto);
connect(SOCK, $paddr) or die "connect: $!";
SOCK->autoflush(1);
print SOCK "list\n"; # the magic word for our "server" script.
my $access = <SOCK>; # access is the response from the server (first line) to
   # a "list" command. e.g: @239.192.0.1 or just "" when uni- or broadcast.
my $need_restart = <SOCK>; # do we need to restart vlc after the program
   # changed? "yes" for dvb, "no" for analog (at least for me)
my @channels = <SOCK>; # one channel name per line
chomp @channels; chomp $access; chomp $need_restart;
$debug && print "access: '$access' need_restart: $need_restart\n";
close (SOCK) or die "close: $!";

$debug && print "numchan: " . (@channels + 0) . "\n";
start_vlc();

$SIG{'PIPE'} = sub{ my $tmp=$SIG{'PIPE'}; $SIG{'PIPE'} = 'IGNORE';
                    print "$me: pipe to vlc broke: $!\nrestarting...\n";
                    close VLC; start_vlc();
                    $SIG{'PIPE'} = $tmp; };
$SIG{'CHLD'} = sub{ my $tmp=$SIG{'CHLD'}; $SIG{'CHLD'} = 'IGNORE';
                    print "$me: vlc died: $!\nrestarting...\n";
                    close VLC; start_vlc();
                    $SIG{'CHLD'} = $tmp; };

my $main = new MainWindow;
#$main->Label(-text => 'Channel Zapper')->pack;
my $top = $main->Frame;
   $top->pack(-side => 'top');#, -ipadx => '0c'); 
my $l1 = $top->Frame;
   $l1->pack(-fill => 'x', -expand => 'yes');
   $l1->Button(-text => 'Quit', -padx => '0.1c',
              -command => sub{ end();$main->destroy }
              )->pack(-side => 'left');
   $l1->Button(-text => '-', -padx => '0.1c',
               -command => sub{ inc_dec_chan(-1); }
               )->pack(-side => 'right');
   $l1->Button(-text => 'Exit Viewer', -padx => '0.05c',
               -command => sub{ quit_vlc(); $main->destroy }
               )->pack; # no -side -> middle (only if it comes last!)
my $l2 = $top->Frame;
   $l2->pack(-fill => 'x', -expand => 'yes');
   $l2->Button(-text => 'Restart', -padx => '0.1c',
               -command => sub{ stop_start_vlc(); }
               )->pack(-side => 'left');
   $l2->Button(-text => '+', -padx => '0.1c',
               -command => sub{ inc_dec_chan(1); }
               )->pack(-side => 'right');
   $l2->Button(-text => 'Record', -padx => '0.1c',
               -command => sub{ start_record(); }
               )->pack; # middle
#my $l3 = $top->Frame;
#   $l3->pack;
my $list = $main->Scrolled (qw/Listbox -setgrid 0 -height 0 -width 0 -scrollbars osow/);
   $list->pack(qw/-side bottom -expand yes -fill both/);
   $list->insert(0, @channels);
   $list->activate(0);
   $list->selectionSet(0);
   $list->bind('<Double-1>'   => sub{do_it ($_[0]->get('active'))});
   $list->bind('<Return>'     => sub{do_it ($_[0]->get('active'))});
   $list->bind('<4>'          => sub{ inc_dec_chan(1); });
   $list->bind('<5>'          => sub{ inc_dec_chan(-1); });
   $list->bind('<plus>'       => sub{ inc_dec_chan(1); });
   $list->bind('<minus>'      => sub{ inc_dec_chan(-1); });
   $list->bind('<KP_Add>'     => sub{ inc_dec_chan(1); });
   $list->bind('<KP_Subtract>'=> sub{ inc_dec_chan(-1); });
   $list->focus;
MainLoop;

sub inc_dec_chan {
  my $ind=$_[0];
  my $skip;
  my $num=$list->curselection;
  $list->selectionClear($num);
  $num+=$ind;
  if ($num > @channels-1){
    $num = @channels-1;
    $skip = 1; # skip=1 -> we have reached one end of the list.
  } elsif ($num < 0){
    $num = 0;
    $skip = 1;
  }
  $list->activate($num);
  $list->selectionSet($num);
  $list->see($num);
  do_it ($list->get('active')) unless defined $skip;
}
  
sub end {
  quit_vlc();
  do_it("exit");
}

sub quit_vlc {
  $SIG{'CHLD'} = 'IGNORE';
  $SIG{'PIPE'} = 'IGNORE';
  print VLC "quit\n";
  close VLC or warn "can't close pipe to vlc: $!";
}

sub stop_start_vlc {
  print VLC "stop\n";
  print VLC "play\n";
}

sub do_it {
  my ($parm) = @_;
  my ($dummy, @dummy);
  $debug && print "sent to socket: $parm\n";
  socket(SOCK, PF_INET, SOCK_STREAM, $proto);
  connect(SOCK, $paddr) or die "connect: $!";
  SOCK->autoflush(1);
  print SOCK "$parm\n";
  @dummy = <SOCK>;
  $debug && print "server response: ";
  $debug && print @dummy;
  close (SOCK)          or die "close: $!";
  if ($parm eq "exit") {
    # print "EXIT!\n";
    exit 0;
    #quit_vlc();
  } elsif ($need_restart ne "no") {
    # needed to reinitialize the stream, pause doesn't do this.
    sleep 1;
    stop_start_vlc();
  }
  $now_transmitted = $parm;
  # print "do_it end reached!\n";
}

sub start_vlc {
  cleanup_ipc(); # maybe vlc died badly. Better clean up.
  open(VLC, "| /usr/local/bin/vlc udp:$access -I rc --fake-tty") or die "can't fork: $!";
  VLC->autoflush(1);
}  

#
# this is ugly, but it seems we have to do it since vlc seems to lose
# shared memory segments if it dies badly or gets killed too hard.
sub cleanup_ipc {
  my @shms;
  my $dummy;
  open(PIPE,"ipcs -m |") or die "can't open pipe to ipcs: $!";
  @shms = <PIPE>;
  close PIPE;
  chomp @shms;
  @shms = grep(/\s0\s*$/, @shms);
  foreach my $shmid (@shms) {
    $shmid =~ m/^\S+\s(\d+)\s/;
    system("ipcrm", "-m", "$1"); # this should be made more nice...
  }
}

sub start_record() {
  $debug && print "now_transmitted = $now_transmitted\n";
  return if ($now_transmitted eq "");
  my $pid;
#  local $SIG{'CHLD'} = 'IGNORE'; # avoid the invasion of the zombies... THIS DOESNT WORK
  my $tmp=$SIG{'CHLD'};
  $SIG{'CHLD'} = sub {print "$me: record_gui exited.\n"}; # avoid the invasion of the zombies...
#  defined ($pid = fork()) or warn "can't fork: $!";
#  if ($pid == 0){
#     $SIG{'CHLD'} = 'IGNORE';
#    exec("/usr/local/bin/vlc-record-gui.pl", "$now_transmitted")
#      or warn "can't exec vlc-record-gui: $!";
#     exit 0;
#  }
  system("/usr/local/bin/vlc-record-gui.pl $now_transmitted &");
  $SIG{'CHLD'}=$tmp;
}
exit 1
