#!/usr/bin/perl -w

use strict;
use Tk;
use Tk::ProgressBar;
use IO::Handle;
use Socket;
#use Time::localtime;

# configure here
my $store = "/tv/dvb";
my $vlc   = "/usr/local/bin/vlc";
my $addr  = 'udp:@239.192.0.1';
#my $addr  = 'udp:@127.0.0.4';
# configure end

my $channel = shift;
my $me = $0;
$me =~ s#.*/##;
die "supply channel-name!" if (! defined $channel);

$SIG{'PIPE'} = sub{ my $tmp=$SIG{'PIPE'}; $SIG{'PIPE'} = 'IGNORE';
                    print "$me: pipe to vlc broke: $!\nrestarting...\n";
#                   close VLC; start_vlc();
                    $SIG{'PIPE'} = $tmp; };
$SIG{'CHLD'} = sub{ my $tmp=$SIG{'CHLD'}; $SIG{'CHLD'} = 'IGNORE';
                    print "$me: vlc died: $!\nrestarting...\n";
#                   close VLC; start_vlc();
                    $SIG{'CHLD'} = $tmp; };

my $elapsed = 0;
my $elapstr = "00:00:00";
my $running = 0;
my $start   = 0;
my $sec = 0;
my $filename = "";

my $main = new MainWindow;
   $main->Label(-text => 'Stream Recorder'
             )->pack;
my $top = $main->Frame;
   $top->pack(qw/-side top/);
   $top->Button(-text => 'Start recording',
               -command => sub{ start_vlc() }
               )->pack(-side => 'left');
   $top->Button(-text => 'Stop/exit',
               -command => sub{ quit_vlc(); $main->destroy }
               )->pack(-side => 'right');
my $mid = $main->Frame;
   $mid->pack;
   $mid->ProgressBar(-variable => \$sec,
               -length => 120, -width => 15,
               -from => 0, -to => 60, -blocks => 6
               )->pack(-side => 'left');
   $mid->Label(-textvariable => \$elapstr)->pack(-side => 'right');
   $mid->repeat(1000, sub{do_running()}); # once a second. enough?
my $bot = $main->Frame;
   $bot->pack(-side => 'bottom');
   $bot->Label(-textvariable => \$filename)->pack;

MainLoop;

sub do_running {
  if ($running == 1) {
    $elapsed = time() - $start;
    $elapstr=val2tm_str($elapsed);
  }
}
sub quit_vlc {
  $SIG{'CHLD'} = 'IGNORE';
  $SIG{'PIPE'} = 'IGNORE';
  # rename the file to reflect the stop time...seconds are not important now.
  my @time = localtime(time);
  my $nowstr = sprintf("%02d%02d", $time[2],$time[1]);
  my $newname = $filename;
  $newname =~ s/(\.mpeg-ts)$/-$nowstr$1/;
  print "newname: $newname\n";
  rename("$store/$filename", "$store/$newname"); 
  print VLC "quit\n";
  close VLC or warn "$me: can't close pipe to vlc: $!";
}

sub start_vlc {
  return if ($running == 1);
  my @time = localtime(time);
  my $nowstr = sprintf("%02d%02d%02d-%02d%02d%02d", 
                        $time[3],$time[4],$time[5]%100,$time[2],$time[1],$time[0]);
  $start  = time();
  $filename = "$channel-$nowstr.mpeg-ts";
  $filename =~ s/ /_/g;

  cleanup_ipc(); # maybe vlc died badly. Better clean up.
  open(VLC, "| $vlc $addr -I rc --fake-tty --sout file/ts:$store/$filename") or die "can't fork: $!";
  VLC->autoflush(1);
  $running=1;
}  

sub val2tm_str {
  my $val = $_[0];
#  my $sec;
  my $min;
  my $hr;
  $sec = $val%60; $val=($val-$sec)/60;
  $min = $val%60; $val=($val-$min)/60;
  $hr  = $val%60;
  return sprintf("%02d:%02d:%02d", $hr,$min,$sec);
}
#
# this is ugly, but it seems we have to do it since vlc seems to lose
# shared memory segments.
sub cleanup_ipc {
  local $SIG{'PIPE'} = sub{ print "$me: SIGPIPE in cleanup_ipc!" }; # debug...
  local $SIG{'CHLD'} = 'IGNORE'; # we get a sigchld when the ipcs finishes...
                                 # this is before the "@shms=<PIPE>"
  my @shms;
  my $dummy;
  open(PIPE,"ipcs -m |") or die "can't open pipe to ipcs: $!";
  @shms = <PIPE>;
  close PIPE;
  chomp @shms;
  @shms = grep(/\s0\s*$/, @shms);
  foreach my $shmid (@shms) {
    $shmid =~ m/^\S+\s(\d+)\s/;
    system("ipcrm", "-m", "$1"); # not nice, one system() with multiple
  }                              # -m parameters would be better. too lazy.
}
exit 1
